import { jsxs, jsx, Fragment } from 'react/jsx-runtime';
import { useState, useEffect, createContext, useContext } from 'react';

const colors = ["#FF38D1", "#8263FF", "#0095FF", "#00B874", "#FF3168", "#FFAB03"];
// Function to get a random color from the array
function getRandomColor() {
  const idx = Math.floor(Math.random() * colors.length);
  const color = colors[idx];
  return color;
}
// CursorChat context
const CursorChatCtx = /*#__PURE__*/createContext(null);
function CursorChat({
  id,
  color = getRandomColor(),
  name = "visitor",
  avatar,
  latency,
  region,
  presence
}) {
  // #region define the cursor state
  // initialize the cursor state
  const [myState, setMyState] = useState({
    id,
    state: "online",
    color,
    x: 0,
    y: 0,
    message: "",
    name,
    avatar,
    latency,
    region
  });
  // initialize other cursors state
  const [otherCursors, setOtherCursors] = useState([]);
  // initialize the connection state
  const [connected, setConnected] = useState(false);
  const [channel, setChannel] = useState(null);
  // #endregion
  // #region initialize the presence connection
  useEffect(() => {
    let channel = null;
    presence.then(yomo => {
      channel = yomo.joinChannel("live-cursor", myState);
      setConnected(true);
      setChannel(channel);
      // hidden user cursor only connected to prscd
      document.body.style.cursor = "none";
    });
    return () => {
      // unsubscribe from the channel
      channel === null || channel === void 0 ? void 0 : channel.leave();
    };
  }, []);
  // #endregion
  // #region initialize the cursor state
  useEffect(() => {
    if (!channel) {
      return;
    }
    // listen to other cursors join
    const unsubscribePeers = channel.subscribePeers(peers => {
      setOtherCursors([...peers]);
    });
    return () => {
      unsubscribePeers();
    };
  }, [channel]);
  // #endregion
  // #region add event listeners to update the cursor state
  useEffect(() => {
    if (!channel) {
      return;
    }
    // listen to other cursors state change
    const updateStateHandler = ({
      payload
    }) => {
      const others = otherCursors.filter(c => c.id !== payload.id);
      setOtherCursors([...others, payload]);
    };
    const unsubscribe = channel.subscribe("update-state", updateStateHandler);
    return () => {
      unsubscribe === null || unsubscribe === void 0 ? void 0 : unsubscribe();
    };
  }, [channel, otherCursors]);
  // #endregion
  // #region add event listeners to update the cursor visibility
  useEffect(() => {
    if (!channel) {
      return;
    }
    // update the cursor state
    const visibilitychangeHandler = () => {
      const newState = Object.assign(Object.assign({}, myState), {
        state: document.hidden ? "away" : "online"
      });
      channel === null || channel === void 0 ? void 0 : channel.broadcast("update-state", newState);
      setMyState(newState);
    };
    document.addEventListener("visibilitychange", visibilitychangeHandler);
    return () => {
      // remove listeners
      document.removeEventListener("visibilitychange", visibilitychangeHandler);
    };
  }, [channel, otherCursors, myState]);
  // #endregion
  // #region add event listeners to update the cursor message
  const [typing, setTyping] = useState(false);
  useEffect(() => {
    // use requestAnimationFrame to throttle the mousemove event
    let needForRAF = true;
    // update the cursor position
    const mousemoveHandler = e => {
      if (needForRAF) {
        needForRAF = false;
        requestAnimationFrame(updateMouseState(e.clientX, e.clientY));
      }
    };
    const updateMouseState = (x, y) => {
      needForRAF = true;
      return () => {
        // if the cursor is not connected, do nothing
        if (!channel) {
          return;
        }
        const newState = Object.assign(Object.assign({}, myState), {
          x,
          y
        });
        channel.broadcast("update-state", newState);
        setMyState(newState);
      };
    };
    document.addEventListener("mousemove", mousemoveHandler);
    return () => {
      document.removeEventListener("mousemove", mousemoveHandler);
    };
  }, [myState, channel]);
  useEffect(() => {
    const keydownHandler = e => {
      if (e.key === "/" || e.key === "Slash") {
        e.preventDefault();
        setTyping(true);
      }
    };
    const keyupHandler = e => {
      if (e.key === "Escape" || e.key === "Enter") {
        setTyping(false);
      }
    };
    document.addEventListener("keydown", keydownHandler);
    document.addEventListener("keyup", keyupHandler);
    return () => {
      document.removeEventListener("keydown", keydownHandler);
      document.removeEventListener("keyup", keyupHandler);
    };
  }, []);
  const onMessageChange = message => {
    const newState = Object.assign(Object.assign({}, myState), {
      message
    });
    channel === null || channel === void 0 ? void 0 : channel.broadcast("update-state", newState);
    setMyState(newState);
  };
  // #endregion
  // #region sync the cursor state to peers
  useEffect(() => {
    if (!channel) {
      return;
    }
    // sync the cursor state to peers
    const unsubscribePeers = channel.subscribePeers(() => {
      channel.broadcast("update-state", myState);
    });
    return () => {
      unsubscribePeers();
    };
  }, [channel, myState]);
  // #endregion
  // #region hide the cursor default style
  useEffect(() => {
    document.body.style.cursor = "none";
    return () => {
      document.body.style.cursor = "auto";
    };
  }, []);
  // #endregion
  // if in SSR, do nothing
  if (typeof window === "undefined") {
    return null;
  }
  if (!connected) {
    return null;
  }
  return jsxs(CursorChatCtx.Provider, Object.assign({
    value: {
      others: otherCursors,
      self: myState
    }
  }, {
    children: [jsx(Cursor, {
      x: myState.x,
      y: myState.y,
      color: myState.color,
      message: myState.message,
      typing: typing,
      name: myState.name,
      avatar: myState.avatar,
      latency: myState.latency,
      region: myState.region,
      onMessageChange: onMessageChange
    }), jsx(OtherCursors, {})]
  }));
}
function OtherCursors() {
  const ctx = useContext(CursorChatCtx);
  if (!ctx) {
    return null;
  }
  const {
    others: cursors
  } = ctx;
  return jsx(Fragment, {
    children: cursors.map(c => jsx(Cursor, {
      x: c.x,
      y: c.y,
      color: c.color,
      name: c.name,
      avatar: c.avatar,
      latency: c.latency,
      region: c.region,
      message: c.message,
      state: c.state
    }, c.id))
  });
}
function Cursor({
  x,
  y,
  color,
  name,
  avatar,
  latency,
  region,
  message,
  state,
  typing = false,
  onMessageChange
}) {
  return jsxs("div", Object.assign({
    style: {
      position: "absolute",
      left: x,
      top: y,
      zIndex: 999999,
      pointerEvents: "none",
      maxWidth: "424px",
      overflow: "hidden",
      whiteSpace: "nowrap",
      textOverflow: "ellipsis",
      display: "-webkit-box",
      WebkitLineClamp: 2,
      WebkitBoxOrient: "vertical",
      opacity: state === "away" ? 0.5 : 1
    }
  }, {
    children: [jsxs("svg", Object.assign({
      width: "20",
      height: "20",
      viewBox: "0 0 23 23",
      fill: color,
      xmlns: "http://www.w3.org/2000/svg"
    }, {
      children: [jsxs("g", Object.assign({
        filter: "url(#filter0_d_142_54)"
      }, {
        children: [jsx("path", {
          fillRule: "evenodd",
          clipRule: "evenodd",
          d: "M2.71004 1.98812C2.42625 1.1935 3.1935 0.426251 3.98812 0.710044L19.6524 6.30442C20.5016 6.60771 20.5472 7.7916 19.7238 8.15927L13.4448 10.9631C13.2194 11.0637 13.0393 11.2441 12.9389 11.4696L10.1582 17.7183C9.79136 18.5427 8.60637 18.4978 8.30287 17.648L2.71004 1.98812Z",
          fill: color
        }), jsx("path", {
          d: "M3.18091 1.81995C3.03902 1.42264 3.42264 1.03902 3.81995 1.18091L19.4842 6.77529C19.9088 6.92694 19.9316 7.51888 19.5199 7.70272L13.2409 10.5065C12.9029 10.6574 12.6326 10.9281 12.4821 11.2663L9.70142 17.515C9.51799 17.9272 8.92549 17.9048 8.77374 17.4799L3.18091 1.81995Z",
          stroke: "white"
        })]
      })), jsx("defs", {
        children: jsxs("filter", Object.assign({
          id: "filter0_d_142_54",
          x: "0.64978",
          y: "0.649767",
          width: "21.6663",
          height: "21.662",
          filterUnits: "userSpaceOnUse",
          colorInterpolationFilters: "sRGB"
        }, {
          children: [jsx("feFlood", {
            floodOpacity: "0",
            result: "BackgroundImageFix"
          }), jsx("feColorMatrix", {
            in: "SourceAlpha",
            type: "matrix",
            values: "0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 127 0",
            result: "hardAlpha"
          }), jsx("feOffset", {
            dy: "2"
          }), jsx("feGaussianBlur", {
            stdDeviation: "1"
          }), jsx("feColorMatrix", {
            type: "matrix",
            values: "0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0.15 0"
          }), jsx("feBlend", {
            mode: "normal",
            in2: "BackgroundImageFix",
            result: "effect1_dropShadow_142_54"
          }), jsx("feBlend", {
            mode: "normal",
            in: "SourceGraphic",
            in2: "effect1_dropShadow_142_54",
            result: "shape"
          })]
        }))
      })]
    })), jsxs("div", Object.assign({
      style: {
        backgroundColor: color,
        borderRadius: message || typing ? "2px 31px 31px 20px" : "18px",
        padding: message || typing ? "10px 20px" : "6px",
        wordWrap: "break-word"
      },
      className: "ml-[20px] text-white"
    }, {
      children: [jsxs("div", Object.assign({
        className: "text-[12px] flex gap-1"
      }, {
        children: [(avatar || name) && jsxs("div", Object.assign({
          className: "rounded-[12px] flex items-center",
          style: {
            background: "rgba(255, 255, 255, 0.1)"
          }
        }, {
          children: [avatar && jsx("img", {
            src: avatar,
            className: "w-6 h-6 rounded-[12px]"
          }), name && jsx("div", Object.assign({
            className: "py-[2px] px-1.5"
          }, {
            children: name
          }))]
        })), (latency || region) && jsxs("div", Object.assign({
          style: {
            background: "rgba(255, 255, 255, 0.1)"
          },
          className: "flex items-center gap-1 rounded-[12px] py-[2px] px-1.5"
        }, {
          children: [jsx(LatencyIcon, {}), latency && jsxs("span", {
            children: [latency, "ms"]
          }), region && jsx("span", {
            children: region
          })]
        }))]
      })), typing ? jsx("input", {
        autoFocus: true,
        className: "bg-transparent border-none outline-none placeholder:text-[rgba(255,255,255,0.6)]",
        value: message,
        onInput: e => onMessageChange === null || onMessageChange === void 0 ? void 0 : onMessageChange(e.target.value),
        placeholder: "Say something"
      }) : jsx("span", Object.assign({
        className: "text-[14px]"
      }, {
        children: message
      }))]
    }))]
  }));
}
function LatencyIcon() {
  return jsxs("svg", Object.assign({
    width: "10",
    height: "11",
    viewBox: "0 0 10 11",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, {
    children: [jsx("circle", {
      cx: "5",
      cy: "5.5",
      r: "4.5",
      stroke: "white"
    }), jsx("path", {
      d: "M4.5 4C4.5 3.72386 4.72386 3.5 5 3.5V3.5C5.27614 3.5 5.5 3.72386 5.5 4V6.5V6.5C4.94772 6.5 4.5 6.05228 4.5 5.5V4Z",
      fill: "white"
    }), jsx("path", {
      d: "M5 6.5C4.72386 6.5 4.5 6.27614 4.5 6V5.5H7C7.27614 5.5 7.5 5.72386 7.5 6C7.5 6.27614 7.27614 6.5 7 6.5H5Z",
      fill: "white"
    })]
  }));
}

export { CursorChat as default };
//# sourceMappingURL=index.esm.js.map
